package CS99;

import java.io.*;

/**
 * This class provides methods that make console I/O easier.
 * @version 2
 * @author Michael Clarkson
 */
public class Console {

	/**
	 * Reference to System.in as a buffered reader.
	 */
	private static BufferedReader in;

	/**
	 * The prompt to use to reprompt for an integer after
	 * an invalid one is entered
	 */
	private static String integerReprompt = "Invalid integer. Try again: ";

	/**
	 * The prompt to use to reprompt for an double after
	 * an invalid one is entered
	 */
	private static String doubleReprompt = "Invalid double. Try again: ";
	
	/**
	 * The prompt to use to reprompt for a character after
	 * an invalid one is entered
	 */
	private static String charReprompt = "Invalid character. Try again: ";

	static {
		// Initialize in
		in = new BufferedReader(new InputStreamReader(System.in));
	}

	/**
	 * Read a string, terminated by newline, from System.in
	 * @return The string that was read
	 */
	public static String readString() {
		String s = null;
		
		try {
			s = in.readLine();
		} catch (Exception e) {
			Utility.abort(e);		
		}
		
		return s;
	}
	
	/**
	 * Read a character, terminated by newline, from System.in
	 * @return The string that was read
	 */
	public static char readChar() {
		char c = 0;
		
		try {
			boolean valid = false;
			do {
				String s = in.readLine();
				if (s.length() == 1) {	// Can only be one character in string
					c = s.charAt(0);
					valid = true;
				} else {
					System.out.print(charReprompt);
				}
			} while (!valid);
		} catch (Exception e) {
			Utility.abort(e);		
		}
		
		return c;
	}

	
	/**
	 * Read an integer, terminated by newline, from System.in.
	 * Keep trying to read until a valid integer is entered.
	 * @return The integer that was read
	 */
	public static int readInt() {
		int i = 0;
		
		try {
			boolean valid = false;
			do {
				try {
					i = Integer.parseInt(in.readLine());
					valid = true;		// no exception occured parsing to an int
				} catch (NumberFormatException e) {
					System.out.print(integerReprompt);
				}
			} while (!valid);
		} catch (Exception e) {
			Utility.abort(e);
		}	
		
		return i;
	}

	/**
	 * Read a double, terminated by newline, from System.in.
	 * Keep trying to read until a valid double is entered.
	 * @return The double that was read
	 */
	public static double readDouble() {
		double d = 0.0;
		
		try {
			boolean valid = true;
			do {
				try {
					d = Double.parseDouble(in.readLine());
					valid = true;		// no exception occured parsing to a double
				} catch (NumberFormatException e) {
					valid = false;
					System.out.print(doubleReprompt);
				}
			} while (!valid);
		} catch (Exception e) {
			Utility.abort(e);
		}	
		
		return d;
	}
	
	/**
	 * Print a message to System.out, then wait for enter to 
	 * be pressed.
	 */
	public static void pause() {
		try {
			System.out.print("Press enter to continue...");
			in.readLine();
		} catch (Exception e) {
			Utility.abort(e);
		}
	}
	
	/**
	 * Sets the prompt that is used to get new input after an invalid
	 * integer is entered
	 * @prompt The prompt to use
	 */
	public static void setIntegerReprompt(String prompt) {
		integerReprompt = prompt;
	}
	
	/**
	 * Sets the prompt that is used to get new input after an invalid
	 * double is entered
	 * @prompt The prompt to use
	 */
	public static void setDoubleReprompt(String prompt) {
		doubleReprompt = prompt;
	}
	
	/**
	 * Sets the prompt that is used to get new input after an invalid
	 * character is entered
	 * @prompt The prompt to use
	 */
	public static void setCharReprompt(String prompt) {
		charReprompt = prompt;
	}

	
}